@@ -273,139 +273,6 @@ package MBX::Alien::FLTK::Base;
                 IGNORE_NUMLOCK    => 1,
                 USE_PROGRESSIVE_DRAW => 1,
                 HAVE_XINERAMA        => 0    # 1.3.x
-
-#~
-#~ /*
-#~  * HAVE_DIRENT_H, HAVE_SYS_NDIR_H, HAVE_SYS_DIR_H, HAVE_NDIR_H,
-#~  * HAVE_SCANDIR, HAVE_SCANDIR_POSIX:
-#~  *
-#~  * Where is <dirent.h> (used only by fl_file_chooser and scandir).
-#~  */
-#~
-#~ #undef HAVE_DIRENT_H
-#~ #undef HAVE_SYS_NDIR_H
-#~ #undef HAVE_SYS_DIR_H
-#~ #undef HAVE_NDIR_H
-#~ #undef HAVE_SCANDIR
-#~ #undef HAVE_SCANDIR_POSIX
-#~
-#~ /*
-#~  * Possibly missing sprintf-style functions:
-#~  */
-#~
-#~ #undef HAVE_VSNPRINTF
-#~ #undef HAVE_SNPRINTF
-#~
-#~ /*
-#~  * String functions and headers...
-#~  */
-#~
-#~ #undef HAVE_STRINGS_H
-#~ #undef HAVE_STRCASECMP
-#~ #undef HAVE_STRLCAT
-#~ #undef HAVE_STRLCPY
-#~
-#~ /*
-#~  * Do we have POSIX locale support?
-#~  */
-#~
-#~ #undef HAVE_LOCALE_H
-#~ #undef HAVE_LOCALECONV
-#~
-#~ /*
-#~  * HAVE_SYS_SELECT_H:
-#~  *
-#~  * Whether or not select() call has its own header file.
-#~  */
-#~
-#~ #undef HAVE_SYS_SELECT_H
-#~
-#~ /*
-#~  * Do we have various image libraries?
-#~  */
-#~
-#~ #undef HAVE_LIBPNG
-#~ #undef HAVE_LIBZ
-#~ #undef HAVE_LIBJPEG
-#~
-#~ /*
-#~  * USE_CAIRO
-#~  *
-#~  * Do we have the cairo library available and want extended cairo use in fltk ?
-#~  * will implies to link cairo.lib in all fltk based apps.
-#~  */
-#~
-#~ #undef USE_CAIRO
-#~
-#~ /*
-#~  * HAVE_CAIRO
-#~  *
-#~  * Do we have the cairo library available?
-#~  */
-#~
-#~ #undef HAVE_CAIRO
-#~
-#~ /*
-#~  * Which header file do we include for libpng?
-#~  */
-#~
-#~ #undef HAVE_PNG_H
-#~ #undef HAVE_LIBPNG_PNG_H
-#~
-#~ /*
-#~  * Do we have the png_xyz() functions?
-#~  */
-#~
-#~ #undef HAVE_PNG_GET_VALID
-#~ #undef HAVE_PNG_SET_TRNS_TO_ALPHA
-#~
-#~ /*
-#~  * Do we have POSIX threading?
-#~  */
-#~
-#~ #undef HAVE_PTHREAD
-#~ #undef HAVE_PTHREAD_H
-#~
-#~ /*
-#~  * Do we have the ALSA library?
-#~  */
-#~
-#~ #undef HAVE_ALSA_ASOUNDLIB_H
-#~
-#~ /*
-#~  * Do we have the long long type?
-#~  */
-#~
-#~ #undef HAVE_LONG_LONG
-#~
-#~ #ifdef HAVE_LONG_LONG
-#~ #  define FLTK_LLFMT	"%lld"
-#~ #  define FLTK_LLCAST	(long long)
-#~ #else
-#~ #  define FLTK_LLFMT	"%ld"
-#~ #  define FLTK_LLCAST	(long)
-#~ #endif /* HAVE_LONG_LONG */
-#~
-#~ /*
-#~  * Do we have the strtoll() function?
-#~  */
-#~
-#~ #undef HAVE_STRTOLL
-#~
-#~ #ifndef HAVE_STRTOLL
-#~ #  define strtoll(nptr,endptr,base) strtol((nptr), (endptr), (base))
-#~ #endif /* !HAVE_STRTOLL */
-#~
-#~ /*
-#~  * Do we have the dlsym() function and header?
-#~  */
-#~
-#~ #undef HAVE_DLFCN_H
-#~ #undef HAVE_DLSYM
-#~
-#~ /*
-#~  * End of "$Id: Base.pm 2fbc10d 2009-09-18 03:50:45Z sanko@cpan.org $".
-#~  */
             }
         );
         {
@@ -508,7 +375,8 @@ int main ( ) {
 }
 
                     if ($exe && `$exe`) {
-                        print($lib ? "$lib\n" : "none required\n");
+                        if   ($lib) { print "$lib\n" }
+                        else        { print "none required\n" }
                         $self->notes(
                              'ldflags' => $self->notes('ldflags') . " $lib ");
                         $_have_pow = 1;
@@ -528,118 +396,6 @@ int main ( ) {
         sub LIBS {
             my ($self) = @_;
             %___LIBS = (
-                fltk_jpeg => {
-                    directory => 'jpeg',
-                    source    => [
-                        map { $_ . '.c' }
-                            qw[jmemnobs jcapimin jcapistd jccoefct jccolor
-                            jcdctmgr jchuff jcinit jcmainct jcmarker jcmaster
-                            jcomapi jcparam jcphuff jcprepct jcsample jctrans
-                            jdapimin jdapistd jdatadst jdatasrc jdcoefct jdcolor
-                            jddctmgr jdhuff jdinput jdmainct jdmarker jdmaster
-                            jdmerge jdphuff jdpostct jdsample jdtrans jerror
-                            jfdctflt jfdctfst jfdctint jidctflt jidctfst jidctint
-                            jidctred jquant1 jquant2 jutils jmemmgr]
-                    ]
-                },
-                fltk_png => {
-                    directory => 'png',
-                    include   => 'zlib',
-                    source    => [
-                        map { $_ . '.c' }
-                            qw[png pngset pngget pngrutil pngtrans pngwutil
-                            pngread pngrio pngwio pngwrite pngrtran pngwtran
-                            pngmem pngerror pngpread]
-                    ]
-                },
-                fltk_z => {
-                    directory => 'zlib',
-                    source    => [
-                        map { $_ . '.c' }
-                            qw[adler32 compress crc32 gzio uncompr deflate
-                            trees zutil inflate inftrees inffast],
-                    ]
-                },
-                fltk => {
-                    directory => 'src',
-                    source    => [
-                        (map { $_ . '.c' }
-                             qw[fl_call_main flstring scandir numericsort
-                             vsnprintf fl_utf]
-                        ),
-                        (map { $_ . '.cxx' }
-                             qw[Fl Fl_Adjuster Fl_Bitmap Fl_Browser
-                             Fl_Browser_ Fl_Browser_load Fl_Box Fl_Button
-                             Fl_Chart Fl_Check_Browser Fl_Check_Button
-                             Fl_Choice Fl_Clock Fl_Color_Chooser Fl_Counter
-                             Fl_Dial Fl_Double_Window Fl_File_Browser
-                             Fl_File_Chooser Fl_File_Chooser2 Fl_File_Icon
-                             Fl_File_Input Fl_Group Fl_Help_View Fl_Image
-                             Fl_Input Fl_Input_ Fl_Light_Button Fl_Menu
-                             Fl_Menu_ Fl_Menu_Bar Fl_Sys_Menu_Bar
-                             Fl_Menu_Button Fl_Menu_Window Fl_Menu_add
-                             Fl_Menu_global Fl_Multi_Label Fl_Overlay_Window
-                             Fl_Pack Fl_Pixmap Fl_Positioner Fl_Preferences
-                             Fl_Progress Fl_Repeat_Button Fl_Return_Button
-                             Fl_Roller Fl_Round_Button Fl_Scroll Fl_Scrollbar
-                             Fl_Shared_Image Fl_Single_Window Fl_Slider
-                             Fl_Tabs Fl_Text_Buffer Fl_Text_Display
-                             Fl_Text_Editor Fl_Tile Fl_Tiled_Image Fl_Tooltip
-                             Fl_Valuator Fl_Value_Input Fl_Value_Output
-                             Fl_Value_Slider Fl_Widget Fl_Window
-                             Fl_Window_fullscreen Fl_Window_hotspot
-                             Fl_Window_iconize Fl_Wizard Fl_XBM_Image
-                             Fl_XPM_Image Fl_abort Fl_add_idle Fl_arg
-                             Fl_compose Fl_display Fl_get_key
-                             Fl_get_system_colors Fl_grab Fl_lock
-                             Fl_own_colormap Fl_visual Fl_x filename_absolute
-                             filename_expand filename_ext filename_isdir
-                             filename_list filename_match filename_setext
-                             fl_arc fl_arci fl_ask fl_boxtype fl_color
-                             fl_cursor fl_curve fl_diamond_box fl_dnd fl_draw
-                             fl_draw_image fl_draw_pixmap fl_encoding_latin1
-                             fl_encoding_mac_roman fl_engraved_label
-                             fl_file_dir fl_font fl_gtk fl_labeltype
-                             fl_line_style fl_open_uri fl_oval_box fl_overlay
-                             fl_overlay_visual fl_plastic fl_read_image
-                             fl_rect fl_round_box fl_rounded_box fl_set_font
-                             fl_set_fonts fl_scroll_area fl_shadow_box
-                             fl_shortcut fl_show_colormap fl_symbols fl_vertex
-                             screen_xywh fl_utf8]
-                        ),
-                        (map { 'xutf8/' . $_ . '.c' }
-                             qw[case is_right2left is_spacing keysym2Ucs
-                             utf8Input utf8Utils utf8Wrap]
-                        )
-                    ]
-                },
-                fltk_forms => {
-                    directory => 'src',
-                    source    => [
-                        map { $_ . '.cxx' }
-                            qw[forms_compatability forms_bitmap forms_free
-                            forms_fselect forms_pixmap forms_timer]
-                    ]
-                },
-                fltk_gl => {
-                    directory => 'src',
-                    source    => [
-                        map { $_ . '.cxx' }
-                            qw[Fl_Gl_Choice Fl_Gl_Overlay Fl_Gl_Window
-                            freeglut_geometry freeglut_stroke_mono_roman
-                            freeglut_stroke_roman freeglut_teapot gl_draw
-                            gl_start glut_compatability glut_font]
-                    ]
-                },
-                fltk_image => {
-                    directory => 'src',
-                    source    => [
-                        map { $_ . '.cxx' }
-                            qw[fl_images_core Fl_BMP_Image Fl_File_Icon2
-                            Fl_GIF_Image Fl_Help_Dialog Fl_JPEG_Image
-                            Fl_PNG_Image Fl_PNM_Image]
-                    ]
-                },
                 fltk2_images => {
                     directory => 'images',
                     source    => [
@@ -742,8 +498,8 @@ int main ( ) {
 
     sub build_fltk {
         my ($self, $build) = @_;
-        my @lib;
-        my @_libs;
+        $self->quiet(1);
+        my (@lib, @_libs);
         if ($self->notes('fltk_branch') eq '1.3.x') {
             @_libs = grep { $_ !~ m[2] } keys %{$self->LIBS};
         }
@@ -815,6 +571,7 @@ int main ( ) {
             print 'Failed to cd to ' . $self->fltk_dir();
             exit 0;
         }
+        $self->quiet(0);
         return @lib ? 1 : 0;
     }
 
@@ -849,7 +606,7 @@ int main ( ) {
                     $self->notes('fltk_branch'),
                     $self->notes('fltk_svn');
                 printf
-                    "Fetching r%s or FLTK's %s branch from %s mirror\n    %s...\n",
+                    "Fetching r%s from the %s branch of fltk from %s mirror\n    %s...\n",
                     $self->notes('fltk_svn'), $self->notes('fltk_branch'),
                     $mirrors{$mirror}, $from;
                 $path = File::Fetch->new(uri => $from)->fetch(to => $dest);
@@ -859,29 +616,32 @@ int main ( ) {
             }
         }
         if (!$path) {
-            printf
-                <<'END', $self->notes('fltk_branch'), $self->notes('fltk_svn'), $dest;
- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
-       ERROR: We failed to fetch fltk-%s-r%s.tar.gz and will exit.
+            my $msg = sprintf <<'END',
+We failed to fetch fltk-%s-r%s.tar.gz.
 
-  If this problem persists, you may download the archive yourself and put
-  it in the ./%s/ directory. Alien::FLTK will attempt to extract and build
-  the libs from there.
+If this problem persists, you may download the archive yourself and put it in
+the ./%s/ directory. Alien::FLTK will attempt to extract and build the libs
+from there.
 
-  Use any of these mirrors:
+Use any of these mirrors:
 
 END
+                $self->notes('fltk_branch'), $self->notes('fltk_svn'), $dest;
             for my $mirror (keys %mirrors) {
-                print " " x 4 . $mirrors{$mirror} . "\n";
+                $msg .= " " x 4 . $mirrors{$mirror} . "\n";
                 for my $prot (qw[ftp http]) {
-                    printf
+                    $msg .= sprintf
                         "      %s://%s/pub/fltk/snapshots/fltk-%s-r%s.tar.gz\n",
                         $prot, $mirror, $self->notes('fltk_branch'),
                         $self->notes('fltk_svn');
                 }
             }
-            print ' ---' x 19;
-            exit 0;    # Clean exit
+            push @{$self->notes('errors')},
+                {stage   => 'fltk source download',
+                 fatal   => 1,
+                 message => $msg
+                };
+            return;
         }
         return $path;
     }
@@ -895,8 +655,12 @@ END
         require Archive::Extract;
         my $ae = Archive::Extract->new(archive => $archive);
         if (!$ae->extract(to => 'src')) {
-            carp "\nError: " . $ae->error;
-            return 0;
+            push @{$self->notes('errors')},
+                {stage   => 'fltk source extraction',
+                 fatal   => 1,
+                 message => $ae->error
+                };
+            return;
         }
         print "okay\n";
         return 1;
@@ -987,16 +751,15 @@ END
                 seek($fh, tell($fh), 0);
                 my $data = $self->notes();
                 if (eval 'require Data::Dump') {
-                    $fh->print(  'do{ my $x = '
-                               . Data::Dump::pp($data)
-                               . "; \$x; }\n");
+                    $fh->print(sprintf 'do{ my $x = %s; $x; }' . "\n",
+                               Data::Dump::pp($data));
                 }
                 else {
                     require Data::Dumper;
-                    $fh->print('do{ my '
-                               . Data::Dumper->new([$data], ['x'])->Purity(1)
-                               ->Dump()
-                               . "\$x; }\n");
+                    my $Dumper = Data::Dumper->new([$data], ['x']);
+                    $Dumper->Purity(1);
+                    $fh->print(sprintf 'do{ my %s; $x; }' . "\n",
+                               $Dumper->Dump());
                 }
                 truncate($fh, tell($fh));
                 $fh->close;
@@ -1012,6 +775,27 @@ END
         return 1;
     }
 
+    sub ACTION_clear_config {
+        my ($self) = @_;
+        print 'Cleaning Alien::FLTK config... ';
+        my $me = _abs($self->base_dir() . '/lib/Alien/FLTK.pm');
+        require IO::File;
+        my $mode_orig = (stat $me)[2] & 07777;
+        chmod($mode_orig | 0222, $me);    # Make it writeable
+        my $fh = IO::File->new($me, 'r+')
+            or die "Can't rewrite $me: $!";
+        seek($fh, 0, 0);
+        while (<$fh>) { last if /^__DATA__$/; }
+        die "Couldn't find __DATA__ token in $me" if eof($fh);
+        seek($fh, tell($fh), 0);
+        $fh->print("do{ my \$x = { }; \$x; }\n");
+        truncate($fh, tell($fh));
+        $fh->close;
+        chmod($mode_orig, $me)
+            or warn "Couldn't restore permissions on $me: $!";
+        print "okay\n";
+    }
+
     sub ACTION_build_fltk {
         my ($self) = @_;
         $self->depends_on('fetch_fltk');
@@ -1057,32 +841,67 @@ END
 
     sub ACTION_code {
         my ($self) = @_;
-        $self->depends_on('fetch_fltk');
-        $self->depends_on('extract_fltk');
-        $self->depends_on('configure_fltk');
-        $self->depends_on('write_config_h');
-        $self->depends_on('build_fltk');
-        $self->depends_on('copy_headers');
+        for my $action (
+            qw[fetch_fltk extract_fltk configure_fltk write_config_h build_fltk
+            copy_headers])
+        {   $self->depends_on($action);
+            $self->dispatch('check_errors');
+        }
         return $self->SUPER::ACTION_code;
     }
+
+    sub ACTION_check_errors {
+        my ($self) = @_;
+        return if !@{$self->notes('errors')};
+        my $fatal = 0;
+        for my $error (@{$self->notes('errors')}) {
+            next if $error->{'seen'}++ && !$error->{'fatal'};
+            $fatal += $error->{'fatal'};
+            my $msg = $error->{'message'};
+            $msg =~ s|(.+)|  $1|gm;
+            printf "\nWARNING: %s error enountered during %s:\n%s\n",
+                ($error->{'fatal'} ? ('*** Fatal') : 'Non-fatal'),
+                $error->{'stage'}, $msg, '-- ' x 10;
+        }
+        if ($fatal) {
+            printf STDOUT ('*** ' x 15) . "\n"
+                . '%s fatal error%s encountered during the build process. '
+                . "Please correct %s and run Build.PL again.\nExiting...",
+                $fatal == 1
+                ? ('A', ' was', 'it')
+                : ($fatal, 's were', 'them');
+            exit 0;
+        }
+    }
+
+    sub ACTION_clean {
+        my $self = shift;
+        $self->dispatch('clear_config');
+        $self->SUPER::ACTION_clean(@_);
+        $self->notes(errors => []);    # Reset fatal and non-fatal errors
+    }
     {
 
         # Ganked from Devel::CheckLib
         sub assert_lib {
-            my ($self, %args) = @_;
+            my ($self, $args) = @_;
             my (@libs, @libpaths, @headers, @incpaths);
 
             # FIXME: these four just SCREAM "refactor" at me
-            @libs = (ref($args{lib}) ? @{$args{lib}} : $args{lib})
-                if $args{lib};
-            @libpaths
-                = (ref($args{libpath}) ? @{$args{libpath}} : $args{libpath})
-                if $args{libpath};
-            @headers = (ref($args{header}) ? @{$args{header}} : $args{header})
-                if $args{header};
-            @incpaths
-                = (ref($args{incpath}) ? @{$args{incpath}} : $args{incpath})
-                if $args{incpath};
+            @libs = (ref($args->{'lib'}) ? @{$args->{'lib'}} : $args->{'lib'})
+                if $args->{'lib'};
+            @libpaths = (ref($args->{'libpath'})
+                         ? @{$args->{'libpath'}}
+                         : $args->{'libpath'}
+            ) if $args->{'libpath'};
+            @headers = (ref($args->{'header'})
+                        ? @{$args->{'header'}}
+                        : $args->{'header'}
+            ) if $args->{'header'};
+            @incpaths = (ref($args->{'incpath'})
+                         ? @{$args->{'incpath'}}
+                         : $args->{'incpath'}
+            ) if $args->{'incpath'};
             my @missing;
 
             # first figure out which headers we can't find ...
@@ -1095,8 +914,8 @@ END
                      lib_dirs     => \@libpaths
                     }
                     );
-                push @missing, $header if !-x $exe;
-                unlink $exe;
+                if   (defined $exe && -x $exe) { unlink $exe }
+                else                           { push @missing, $header }
             }
 
             # now do each library in turn with no headers
@@ -1109,8 +928,8 @@ END
                                      extra_linker_flags => "-l$lib"
                                     }
                     );
-                push @missing, $lib if !-x $exe;
-                unlink $exe;
+                if   (defined $exe && -x $exe) { unlink $exe }
+                else                           { push @missing, $lib }
             }
             my $miss_string = join(q{, }, map {qq{'$_'}} @missing);
             if (@missing) {
