@@ -1,19 +1,25 @@
-package inc::MBX::Alien::FLTK::Platform::Unix::Darwin;
+package MBX::Alien::FLTK::Platform::Unix::Darwin;
 {
     use strict;
     use warnings;
     use Carp qw[];
     use Config qw[%Config];
-    use lib '../../../../../../';
-    use inc::MBX::Alien::FLTK::Utility qw[_o _a _rel _abs can_run];
-    use inc::MBX::Alien::FLTK;
-    use base 'inc::MBX::Alien::FLTK::Platform::Unix';
+    use lib qw[.. ../../../../..];
+    use MBX::Alien::FLTK::Utility
+        qw[_o _a _dir _rel _abs find_h find_lib can_run];
+    use MBX::Alien::FLTK;
+    use base 'MBX::Alien::FLTK::Platform::Unix';
     $|++;
 
     sub configure {
         my ($self) = @_;
-        $self->SUPER::configure(qw[no_gl no_x11]) || return 0;
-        $self->notes('define')->{'USE_QUARTZ'} = 1;    # Alpha
+        $self->SUPER::configure(qw[no_gl no_x11])
+            || return 0;    # Get basic config data
+        print "Gathering Solaris specific configuration data...\n";
+
+        # Asssumed true since this is *nix
+        print "have pthread... yes (assumed)\n";
+        $self->notes('config')->{'HAVE_PTHREAD'} = 1;
         $self->notes(
                ldflags => ' -framework Carbon -framework ApplicationServices '
                    . $self->notes('ldflags'));
@@ -45,6 +51,6 @@ Creative Commons Attribution-Share Alike 3.0 License. See
 http://creativecommons.org/licenses/by-sa/3.0/us/legalcode.  For
 clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 
-=for git $Id: Darwin.pm dfa6aa4 2010-01-16 21:12:51Z sanko@cpan.org $
+=for git $Id: Darwin.pm a97e5a4 2009-10-25 14:34:56Z sanko@cpan.org $
 
 =cut
